@@ -16,6 +16,7 @@
 import urllib.parse
 from time import sleep
 from lxml import etree
+import time
 
 
 class Events:
@@ -171,47 +172,86 @@ class Events:
         encode_data = encode_multipart_formdata(data)
         data = encode_data[0]
         header['Content-Type'] = encode_data[1]
-        return bool(int(str(self.con.post(path="upLoadFile", headers=header, data=data).read(), 'utf-8')))
+        return bool(int(str(self.device.con.post(path="upLoadFile", headers=header, data=data).read(), 'utf-8')))
 
     def file_exist(self, file_path):
-        return bool(int(str(self.con.get(path="checkFileExist", args="filename=" + file_path).read(), 'utf-8')))
+        return bool(int(str(self.device.con.get(path="checkFileExist", args="filename=" + file_path).read(), 'utf-8')))
 
     def file_remove(self, file_path):
-        return bool(int(str(self.con.get(path="fileRemove", args="filename=" + file_path).read(), 'utf-8')))
+        return bool(int(str(self.device.con.get(path="fileRemove", args="filename=" + file_path).read(), 'utf-8')))
 
     def file_move(self, source_path, target_path):
-        return bool(int(str(self.con.get(path="fileMove", args="source=" + source_path + "&target=" + target_path).read(), 'utf-8')))
+        return bool(int(str(self.device.con.get(path="fileMove", args="source=" + source_path + "&target=" + target_path).read(), 'utf-8')))
 
     def file_copy(self, source_path, target_path):
-        return bool(int(str(self.con.get(path="fileCopy", args="source=" + source_path + "&target=" + target_path).read(), 'utf-8')))
+        return bool(int(str(self.device.con.get(path="fileCopy", args="source=" + source_path + "&target=" + target_path).read(), 'utf-8')))
 
     def dir_exist(self, dir_path):
         return self.file_exist(dir_path)
 
     def mkdir(self, dir_path):
-        return bool(int(str(self.con.get(path="mkdir", args="dirname=" + dir_path).read(), 'utf-8')))
+        return bool(int(str(self.device.con.get(path="mkdir", args="dirname=" + dir_path).read(), 'utf-8')))
 
     def is_installed(self, sopid):
-        return bool(int(str(self.con.get(path="isAppInstalled", args="sopid=" + sopid).read(), 'utf-8')))
+        return bool(int(str(self.device.con.get(path="isAppInstalled", args="sopid=" + sopid).read(), 'utf-8')))
 
     def is_uninstallable(self, sopid):
-        return bool(int(str(self.con.get(path="isAppUninstallable", args="sopid=" + sopid).read(), 'utf-8')))
+        return bool(int(str(self.device.con.get(path="isAppUninstallable", args="sopid=" + sopid).read(), 'utf-8')))
 
     def install(self, file_path):
         if self.upload_file(file_path, "/tmp/"):
             file_name = file_path.split("/")[len(file_path.split("/")) - 1]
-            self.con.get(path="install", args="filepath=/tmp/" + file_name)
+            self.device.con.get(path="install", args="filepath=/tmp/" + file_name)
             return True
         return False
 
     def uninstall(self, sopid):
-        return bool(int(str(self.con.get(path="uninstall", args="sopid=" + sopid).read(), 'utf-8')))
+        return bool(int(str(self.device.con.get(path="uninstall", args="sopid=" + sopid).read(), 'utf-8')))
 
     def system_time(self):
-        return int(str(self.con.get(path="getDatetime").read(), 'utf-8'))
+        return int(str(self.device.con.get(path="getDatetime").read(), 'utf-8'))
 
     def latest_toast(self):
-        return str(self.con.get(path="getLatestToast").read(), 'utf-8')
+        return str(self.device.con.get(path="getLatestToast").read(), 'utf-8')
 
     def clear_app_data(self, sopid):
         return self.__reply_status_check(self.device.con.get(path="clearAppData", args="sopid=" + sopid))
+
+    def get_panel_state(self):
+        return bool(int(str(self.device.con.get(path="getPanelState").read(), 'utf-8')))
+
+    def set_panel_open(self):
+        if not self.get_panel_state():
+            return self.__reply_status_check(self.device.con.get(path="setPanelState"))
+
+    def set_panel_close(self):
+        if self.get_panel_state():
+            return self.__reply_status_check(self.device.con.get(path="setPanelState"))
+
+    def launch(self, sopid, uiappid, timeout=None):
+        self.device.refresh_layout()
+        self.device.con.get(path="launchApp", args="sopid=" + sopid + "&" + "uiappid=" + uiappid)
+        if timeout is None:
+            timeout = self.device.default_timeout
+        die_time = int(time.time()) + timeout
+        while int(time.time()) < die_time:
+            self.device.refresh_layout()
+            selector = etree.XML(self.device.xmlStr.encode('utf-8'))
+            if selector.get("sopId") == sopid:
+                return True
+            self.device.con.get(path="launchApp", args="sopid=" + sopid + "&" + "uiappid=" + uiappid)
+            sleep(1)
+        return False
+
+    def close(self, sopid):
+        return self.__reply_status_check(self.device.con.get(path="quitApp", args="sopid=" + sopid))
+
+    def is_topmost(self, sopid):
+        self.device.refresh_layout()
+        selector = etree.XML(self.device.xmlStr.encode('utf-8'))
+        if selector.get("sopId") == sopid:
+            return True
+        return False
+
+    def clear_data(self, sopid):
+        return self.device.clear_app_data(sopid)
