@@ -44,22 +44,19 @@ class WatchWorker:
                 for watcher in self.__watcher_list:
                     flag = 0
                     for xpath in watcher['xpath_list']:
-                        a = self.device.application(xpath['sop_id'], '')
-                        if a.is_topmost():
-                            if a.find_item_by_xpath(xpath['xpath'], 1).exist():
-                                flag += 1
+                        if self.device.find_item_by_xpath(xpath['sop_id'], xpath['xpath'], 1).exist():
+                            flag += 1
                     if flag == len(watcher['xpath_list']):
                         for xpath in watcher['xpath_list']:
                             if xpath['index'] == watcher['index']:
                                 if watcher['active'] == 'click':
-                                    a = self.device.application(xpath['sop_id'], '')
-                                    a.find_item_by_xpath(xpath['xpath'], 1).click()
+                                    self.device.find_item_by_xpath(xpath['sop_id'], xpath['xpath'], 1).click()
                                 elif watcher['active'] == 'back':
                                     self.device.back()
                                 elif watcher['active'] == 'home':
                                     self.device.home()
                                 elif watcher['active'] == 'launch':
-                                    self.device.application( watcher['active_sop_id'], watcher['active_ui_app_id']).launch()
+                                    self.device.launch(watcher['active_sop_id'], watcher['active_ui_app_id'])
             except Exception as e:
                 print(e)
                 main_process.resume()
